use std::path::Path;

use crate::Component;

use super::cuddle_vars::{load_cuddle_file, CuddleVariable, CuddleVariables};

pub struct Ingress {
    variables: CuddleVariables,
}

impl Ingress {
    pub async fn new(path: &Path) -> anyhow::Result<Self> {
        let variables = load_cuddle_file(path).await?;

        Ok(Self { variables })
    }
}

impl Component for Ingress {
    fn name(&self) -> String {
        format!("cuddle/ingress")
    }

    fn render(
        &self,
        environment: &str,
        value: &serde_yaml::Value,
    ) -> Option<anyhow::Result<(String, String)>> {
        if let Some(true) = self
            .variables
            .0
            .get("database")
            .and_then(|v| match v {
                CuddleVariable::Object(o) => Some(o),
                _ => None,
            })
            .and_then(|o| o.0.get("crdb"))
            .and_then(|o| match o {
                CuddleVariable::String(o) => {
                    if o == "true" {
                        Some(true)
                    } else {
                        None
                    }
                }
                _ => None,
            })
        {
            return Some(Ok((
                format!("{}.yaml", self.name().replace("/", "-")),
                r#"
apiVersion: v1
kind: ConfigMap
metadata:
  name: {{ vars.cuddle_crdb.file_name(vars.cuddle_vars.service) }}
  namespace:  {{ vars.cluster_vars.namespace }}
data:
  DATABASE_URL: postgresql://root@{{environment}}-cluster:26257/{{ vars.cuddle_vars.service | replace("-", "_") }}
                        "#
                .into(),
            )));
        }

        None
    }
}
